"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.XrplKmsService = void 0;
const aws_sdk_1 = require("aws-sdk");
const asn1 = __importStar(require("asn1.js"));
const bn_js_1 = __importDefault(require("bn.js"));
const hashjs = __importStar(require("hash.js"));
const ripple_binary_codec_1 = require("ripple-binary-codec");
const hashes_1 = require("xrpl/dist/npm/utils/hashes");
const ripple_keypairs_1 = require("ripple-keypairs");
const Signature = require("elliptic/lib/elliptic/ec/signature");
const EcdsaSigAsnParse = asn1.define('EcdsaSig', function () {
    this.seq().obj(this.key('r').int(), this.key('s').int());
});
const EcdsaPubKey = asn1.define('EcdsaPubKey', function () {
    this.seq().obj(this.key('algo').seq().obj(this.key('a').objid(), this.key('b').objid()), this.key('pubKey').bitstr());
});
class XrplKmsService {
    constructor(awsAccessKey, awsSecretKey, awsRegion, kmsKeyId) {
        this.kmsKeyId = kmsKeyId;
        this.kms = new aws_sdk_1.KMS({
            accessKeyId: awsAccessKey,
            secretAccessKey: awsSecretKey,
            region: awsRegion,
            apiVersion: "2014-11-01",
        });
    }
    sign(msg) {
        return __awaiter(this, void 0, void 0, function* () {
            const params = {
                KeyId: this.kmsKeyId,
                Message: msg,
                SigningAlgorithm: "ECDSA_SHA_256",
                MessageType: "DIGEST",
            };
            const res = yield this.kms.sign(params).promise();
            if (res.Signature == undefined) {
                throw new Error("Signature is undefined.");
            }
            const decoded = EcdsaSigAsnParse.decode(res.Signature, "der");
            const r = decoded.r;
            let s = decoded.s;
            const secp256k1N = new bn_js_1.default("fffffffffffffffffffffffffffffffebaaedce6af48a03bbfd25e8cd0364141", 16);
            const secp256k1halfN = secp256k1N.div(new bn_js_1.default(2));
            if (s.gt(secp256k1halfN)) {
                s = secp256k1N.sub(s);
            }
            return { r, s };
        });
    }
    getUncompressedPublicKey() {
        return __awaiter(this, void 0, void 0, function* () {
            const pubKey = yield this.kms
                .getPublicKey({
                KeyId: this.kmsKeyId,
            })
                .promise();
            const publicKeyBuffer = pubKey.PublicKey;
            const res = EcdsaPubKey.decode(publicKeyBuffer, "der");
            return res.pubKey.data;
        });
    }
    getCompressedPublicKey() {
        return __awaiter(this, void 0, void 0, function* () {
            const uncompressed = yield this.getUncompressedPublicKey();
            const header = parseInt(uncompressed.toString("hex").slice(uncompressed.length * 2 - 2, uncompressed.length * 2), 16) % 2 ? "03" : "02";
            return Buffer.from(header + uncompressed.toString("hex").slice(2, 66), "hex");
        });
    }
    getXrpAddress() {
        return __awaiter(this, void 0, void 0, function* () {
            const publicKey = yield this.getCompressedPublicKey();
            return (0, ripple_keypairs_1.deriveAddress)(publicKey.toString("hex"));
        });
    }
    signXrpTransaction(transaction) {
        return __awaiter(this, void 0, void 0, function* () {
            const publicKey = yield this.getCompressedPublicKey();
            const txToSignAndEncode = Object.assign({}, transaction);
            txToSignAndEncode.SigningPubKey = publicKey.toString("hex");
            const encodedForSigning = (0, ripple_binary_codec_1.encodeForSigning)(txToSignAndEncode);
            const encodedHash = hashjs.sha512().update(Buffer.from(encodedForSigning, "hex")).digest().slice(0, 32);
            const { r, s } = yield this.sign(Buffer.from(encodedHash));
            const signatureBytes = (new Signature({ r: r.toString(16), s: s.toString(16) })).toDER();
            const signature = Buffer.from(signatureBytes).toString("hex");
            txToSignAndEncode.TxnSignature = signature;
            const serialized = (0, ripple_binary_codec_1.encode)(txToSignAndEncode);
            return {
                payload: serialized,
                hash: (0, hashes_1.hashSignedTx)(serialized),
            };
        });
    }
}
exports.XrplKmsService = XrplKmsService;
//# sourceMappingURL=XrplKmsService.js.map